<?php

namespace Drupal\civicrm_form_builder_blocks\EventSubscriber;

use Drupal\block\BlockRepositoryInterface;
use Drupal\civicrm\Civicrm;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Class CivicrmLoadResourcesSubscriber EventSubscriber.
 */
class CivicrmLoadResourcesSubscriber implements EventSubscriberInterface {

  /**
   * The CiviCRM service.
   *
   * @var \Drupal\civicrm\Civicrm
   */
  private $civicrm;

  /**
   * The block repository service.
   *
   * @var \Drupal\block\BlockRepositoryInterface
   */
  private $blockRepository;

  /**
   * CivicrmLoadResourcesSubscriber constructor.
   *
   * @param \Drupal\civicrm\Civicrm $civicrm
   *   The CiviCRM Service.
   * @param \Drupal\block\BlockRepositoryInterface $block_repository
   *   The block repository service.
   */
  public function __construct(Civicrm $civicrm, BlockRepositoryInterface $block_repository) {
    $this->civicrm = $civicrm;
    $this->blockRepository = $block_repository;
  }

  /**
   * Check for blocks or our type rendering on a page.
   *
   * @param \Symfony\Component\HttpKernel\Event\RequestEvent $event
   *   The request event.
   */
  public function checkForResourceLoad(RequestEvent $event) {
    $names_to_load = [];
    $page_blocks = $this->blockRepository->getVisibleBlocksPerRegion();
    foreach ($page_blocks as $region => $region_blocks) {
      if (!empty($region_blocks)) {
        foreach ($region_blocks as $block_id => $block) {
          if (get_class($block->getPlugin()) == 'Drupal\civicrm_form_builder_blocks\Plugin\Block\AfformBlock') {
            $plugin = $block->getPlugin();
            $config = $plugin->getConfiguration();
            if (!empty($config['afform_name'])) {
              $names_to_load[] = $config['afform_name'];
            }
          }
        }
      }
    }
    if (!empty($names_to_load)) {
      $this->civicrm->initialize();
      \CRM_Core_Resources::singleton()->addCoreResources();
      foreach ($names_to_load as $name) {
        \Civi::service('angularjs.loader')->addModules($name);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[KernelEvents::REQUEST][] = ['checkForResourceLoad'];
    return $events;
  }

}
