<?php

namespace Drupal\civicrm_form_builder_blocks\EventSubscriber;

use Drupal\civicrm\Civicrm;
use Drupal\Core\Block\BlockPluginInterface;
use Drupal\layout_builder\Event\SectionComponentBuildRenderArrayEvent;
use Drupal\layout_builder\LayoutBuilderEvents;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Determines if block of our type is being rendered in a layout.
 */
class LayoutBuilderSubscriber implements EventSubscriberInterface {

  /**
   * The CiviCRM service.
   *
   * @var \Drupal\civicrm\Civicrm
   */
  private $civicrm;

  /**
   * Creates a LayoutBuilderSubscriber object.
   *
   * @param \Drupal\civicrm\Civicrm $civicrm
   *   The CiviCRM Service.
   */
  public function __construct(Civicrm $civicrm) {
    $this->civicrm = $civicrm;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[LayoutBuilderEvents::SECTION_COMPONENT_BUILD_RENDER_ARRAY] = [
      'onBuildRender',
      9999,
    ];
    return $events;
  }

  /**
   * Load Civi JS if a form builder block is being rendered.
   *
   * @param \Drupal\layout_builder\Event\SectionComponentBuildRenderArrayEvent $event
   *   The section component render event.
   */
  public function onBuildRender(SectionComponentBuildRenderArrayEvent $event) {
    $block = $event->getPlugin();
    if (!$block instanceof BlockPluginInterface) {
      return;
    }
    if (get_class($block) != 'Drupal\civicrm_form_builder_blocks\Plugin\Block\AfformBlock') {
      return;
    }
    $block_config = $block->getConfiguration();

    if (!empty($block_config['afform_name'])) {
      // This event fires before the AfformBlock plugin code.
      // This doesn't work.
      /*$build = $event->getBuild();
      $build['#cache']['max-age'] = 0;
      $event->setBuild($build);*/
      $loaded = &drupal_static(__FUNCTION__);
      if (!isset($loaded)) {
        $this->civicrm->initialize();
        \CRM_Core_Resources::singleton()->addCoreResources();
        $loaded = 1;
      }
      \Civi::service('angularjs.loader')->addModules($block_config['afform_name']);
    }
  }

}
