<?php

namespace Drupal\civicrm_form_builder_blocks\Plugin\Block;

use Civi\Api4\Afform;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a 'AfformBlock' block.
 *
 * @Block(
 *  id = "civicrm_form_builder_block",
 *  admin_label = @Translation("CiviCRM Form Builder block"),
 * )
 */
class AfformBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * Drupal\civicrm\Civicrm definition.
   *
   * @var \Drupal\civicrm\Civicrm
   */
  protected $civicrm;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = new static($configuration, $plugin_id, $plugin_definition);
    $instance->civicrm = $container->get('civicrm');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $build = [];
    $this->civicrm->initialize();
    $config = $this->getConfiguration();
    $afform = civicrm_api4('Afform', 'get', [
      'select' => ['directive_name', 'module_name', 'title'],
      'where' => [['name', '=', $config['afform_name']]],
    ])->first();
    if ($afform) {
      $build['#theme'] = 'afform_block';
      $build['#module_name'] = $afform['module_name'];
      $build['#directive_name'] = $afform['directive_name'];
      $build['#afform_title'] = $afform['title'];
    }
    else {
      $build['#markup'] = '';
    }

    /*$build['#cache'] = [
    'tags' => ['afform:' . $config['afform_name']],
    ];*/
    // Do not set cache, blocks in layout builder don't work otherwise.
    // That is the code to load the Civi Resources and block Angular JS.
    // If we figure out different way to detect,
    // If a layout builder block is rendered,
    // Then we can stop setting no cache.
    $build['#cache']['max-age'] = 0;

    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);

    $config = $this->getConfiguration();
    $this->civicrm->initialize();

    $afform_options = [];
    $afforms = Afform::get(FALSE)
      ->addWhere('type', '!=', 'block')
      ->addSelect('name', 'type', 'title')
      ->execute();
    foreach ($afforms as $afform) {
      $afform_options[$afform['name']] = (!empty($afform['title']) ? $afform['title'] : $afform['name']) . ' (' . $afform['type'] . ')';
    }

    $form['afform_name'] = [
      '#type' => 'select',
      '#options' => $afform_options,
      '#description' => $this->t('Choose the CiviCRM Afform to embed for this block.'),
      '#title' => $this->t('Form Name'),
      '#default_value' => $config['afform_name'] ?? '',
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $this->configuration['afform_name'] = $form_state->getValue('afform_name');
  }

  /**
   * {@inheritdoc}
   */
  public function getDefaultConfiguration() {
    return [
      'afform_name' => '',
    ] + parent::defaultConfiguration();
  }

}
